@@ -160,14 +160,14 @@ let pack3 t = map t ~f:(fun f x y z -> f (x, y, z))
 include Ast_pattern_generated
 
 let echar      t = pexp_constant (pconst_char   t     )
-let estring    t = pexp_constant (pconst_string t drop)
+let estring    t = pexp_constant (pconst_string t drop drop)
 let efloat     t = pexp_constant (pconst_float  t drop)
 
 let pchar      t = ppat_constant (pconst_char   t     )
-let pstring    t = ppat_constant (pconst_string t drop)
+let pstring    t = ppat_constant (pconst_string t drop drop)
 let pfloat     t = ppat_constant (pconst_float  t drop)
 
-let int'       (T f) = T (fun ctx loc x k -> f ctx loc (Int.of_string       x) k)
+let int'       (T f) = T (fun ctx loc x k -> f ctx loc (int_of_string       x) k)
 let int32'     (T f) = T (fun ctx loc x k -> f ctx loc (Int32.of_string     x) k)
 let int64'     (T f) = T (fun ctx loc x k -> f ctx loc (Int64.of_string     x) k)
 let nativeint' (T f) = T (fun ctx loc x k -> f ctx loc (Nativeint.of_string x) k)
